package com.qunzhixinxi.hnqz.admin.api.dto;

import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.constant.enums.LockEnum;
import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;

/**
 * 执行计划dto
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 11:16
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class SysImplementPlanDTO {

    @Data
    public static class OnCreate {

        /**
         * 计划名称
         */
        @NotBlank(message = "计划名称必填")
        private String planName;

        /**
         * 计划类型
         */
        @NotNull(message = "计划类型必填")
        private UpmsType.ImplementPlanType planType;

        /**
         * 父ID
         */
        @NotNull(message = "关联计划ID必填")
        private Integer parentId;

        /**
         * 计划值
         */
        @NotNull(message = "计划值必填")
        @Min(value = 0L, message = "计划值必须大于{value}")
        private Long planScore;

        /**
         * 产品ID
         */
        @NotNull(message = "产品ID必填")
        private Integer skuId;

        /**
         * 受托人ID
         */
        @NotNull(message = "受托人ID必填")
        private Long consigneeId;

        /**
         * 签发时间
         */
        @NotNull(message = "签发时间必填")
        private LocalDate planIssue;

        /**
         * 截止日期
         */
        @NotNull(message = "截止日期必填")
        private LocalDate planExpiry;

        /**
         * 锁定标记
         */
        @NotNull(message = "锁定标记必填")
        private LockEnum locFlag;

    }

    @Data
    public static class OnPage {

        /**
         * 计划名称
         */
        private String planName;

        /**
         * 审核状态
         */
        private UpmsState.ImplementPlanState planState;

        /**
         * 受托人ID
         */
        private Integer consigneeId;

        /**
         * 周期
         */
        private LocalDate[] period;


        @NotNull(message = "页码必填")
        @Min(value = 1, message = "页码最小为{value}")
        private Integer current;

        @NotNull(message = "分页数量必填")
        @Min(value = 10, message = "分页数量最小为{value}")
        @Max(value = 100, message = "分页数量最大为{value}")
        private Integer size;
    }

    @Data
    public static class OnCheck {

        @NotNull(message = "计划ID必填")
        private Integer planId;

        @NotNull(message = "审核结果必填")
        private Boolean result;

        @NotBlank(message = "审核意见必填")
        private String msg;
    }
}
