package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.dto.SysImplementPlanDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.SysImplementPlanDetailsDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplementPlan;
import com.qunzhixinxi.hnqz.admin.api.vo.SysImplementPlanVO;
import com.qunzhixinxi.hnqz.admin.manager.SysImplementPlanManager;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 执行计划前端控制器
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 11:42
 */
@RestController
@AllArgsConstructor
public class SysImplementPlanController {

    private final SysImplementPlanManager implementPlanManager;


    @SysLog(value = "创建执行计划")
    @PostMapping(value = "/plan/create")
    public R<Boolean> createPlan(@Validated @RequestBody SysImplementPlanDTO.OnCreate resource) {

        return R.ok(implementPlanManager.createPlan(resource, SecurityUtils.getUser()));

    }


    @GetMapping(value = "/plan/page")
    public R<Page<?>> pagePlans(@Validated SysImplementPlanDTO.OnPage query) {

        return R.ok(implementPlanManager.pagePlans(query, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/plan/list")
    public R<List<SysImplementPlanVO.ToList>> listPlans(@RequestParam(value = "planState", required = false, defaultValue = "PASSED") UpmsState.ImplementPlanState planState) {
        return R.ok(implementPlanManager.listPlans(planState, SecurityUtils.getUser()));
    }


    @SysLog(value = "审核计划")
    @PostMapping(value = "/plan/check")
    public R<Boolean> checkPlans(@Validated @RequestBody SysImplementPlanDTO.OnCheck resource) {

        return R.ok(implementPlanManager.checkPlan(resource, SecurityUtils.getUser()));
    }


    @SysLog(value = "创建计划详情")
    @PostMapping(value = "/plan/details/create")
    public R<Boolean> createDetails(@Validated @RequestBody SysImplementPlanDetailsDTO.OnCreate resource) {
        return R.ok(implementPlanManager.createPlanDetails(resource, SecurityUtils.getUser()));
    }

    @SysLog(value = "审核计划详情")
    @PostMapping(value = "/plan/details/check")
    public R<Boolean> checkPlanDetails(@Validated @RequestBody SysImplementPlanDetailsDTO.OnCheck resource) {

        return R.ok(implementPlanManager.checkPlanDetails(resource, SecurityUtils.getUser()));
    }

    @GetMapping(value = "/plan/details/{details_id}")
    public R<Map<String, Object>> getPlanDetails(@PathVariable(value = "details_id") Integer detailsId) {
        return R.ok(implementPlanManager.getPlanDetails(detailsId));
    }


}
