package com.qunzhixinxi.hnqz.admin.manager;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.dto.SysImplementPlanDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.SysImplementPlanDetailsDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplPlanDetails;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplPlanDetailsItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplementPlan;
import com.qunzhixinxi.hnqz.admin.api.entity.SysPlanPkg;
import com.qunzhixinxi.hnqz.admin.api.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.api.vo.SysImplementPlanVO;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysImplPlanDetailsItemService;
import com.qunzhixinxi.hnqz.admin.service.SysImplPlanDetailsService;
import com.qunzhixinxi.hnqz.admin.service.SysImplementPlanService;
import com.qunzhixinxi.hnqz.admin.service.SysPlanPkgService;
import com.qunzhixinxi.hnqz.admin.service.SysRoleService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.common.core.exception.BizException;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 执行计划管理器
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 11:44
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysImplementPlanManager {

    private final SysImplementPlanService implementPlanService;
    private final SysImplPlanDetailsService implPlanDetailsService;
    private final SysDeptService deptService;
    private final SysRoleService roleService;
    private final SysImplPlanDetailsItemService implPlanDetailsItemService;
    private final WmDaDrugEntDrugtableService drugEntDrugtableService;
    private final SysPlanPkgService planPkgService;
    private final WmScorePackageService scorePackageService;

    /**
     * 创建计划
     *
     * @param resource 计划信息
     * @param operator 操作人
     * @return 创建结果
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean createPlan(SysImplementPlanDTO.OnCreate resource, HnqzUser operator) {

        String username = operator.getUsername();
        LocalDateTime now = LocalDateTime.now();
        SysImplementPlan plan = BeanUtil.copyProperties(resource, SysImplementPlan.class);
        plan.setPlanState(UpmsState.ImplementPlanState.INIT);
        plan.setConsignorId(operator.getDeptId());
        plan.setCreateBy(username);
        plan.setUpdateBy(username);
        plan.setCreateTime(now);
        plan.setUpdateTime(now);

        if (resource.getParentId() != 0) {

            Long availScore = implementPlanService.calcAvailScore(resource.getParentId(), resource.getPlanScore());
            SysImplementPlan parentPlan = new SysImplementPlan();
            parentPlan.setPlanId(resource.getParentId());
            parentPlan.setAvailScore(availScore);
            parentPlan.setUpdateBy(username);
            parentPlan.setUpdateTime(now);
            implementPlanService.updatePlan(parentPlan);
        }

        plan.setAvailScore(resource.getPlanScore());

        return implementPlanService.createPlan(plan);
    }


    /**
     * 获取计划分页
     *
     * @param query    查询条件
     * @param operator 操作人
     * @return 分页结果
     */
    public Page<?> pagePlans(SysImplementPlanDTO.OnPage query, HnqzUser operator) {

        int entLevel = roleService.getEntLevel();


        // 获取根目录分页
        Page<SysImplementPlan> page = this.getPage(operator, entLevel, query.getCurrent(), query.getSize(), query.getPlanName(),
                query.getConsigneeId(), operator.getDeptId(), query.getPeriod());

        List<SysImplementPlan> records = page.getRecords();

        if (CollUtil.isEmpty(records)) {
            return page;
        }

        // 查询出子数据
        Set<Integer> pIds = records.stream().map(SysImplementPlan::getPlanId).collect(Collectors.toSet());

        // 区域的预算
        if (CollUtil.isNotEmpty(pIds)) {
            List<SysImplementPlan> subPlans = implementPlanService.list(Wrappers.<SysImplementPlan>lambdaQuery().in(SysImplementPlan::getParentId, pIds));

            if (CollUtil.isNotEmpty(subPlans)) {
                records.addAll(subPlans);

                // 服务商的预算
                Set<Integer> finalSubPlanIds = subPlans.stream().mapToInt(SysImplementPlan::getParentId).boxed().collect(Collectors.toSet());

                if (CollUtil.isNotEmpty(finalSubPlanIds)) {
                    List<SysImplementPlan> finalSubPlans = implementPlanService.list(Wrappers.<SysImplementPlan>lambdaQuery().in(SysImplementPlan::getParentId, finalSubPlanIds));
                    records.addAll(finalSubPlans);
                }
            }
        }

        // 获取承接人ID
        Set<Integer> consigneeIds = records.stream().mapToInt(SysImplementPlan::getConsigneeId).boxed().collect(Collectors.toSet());

        // 获取承接人
        Map<Integer, SysDept> consigneeMap = deptService.mapConsignee(consigneeIds);

        // 获取商品信息

        Set<Integer> skuIds = records.stream().map(SysImplementPlan::getSkuId).collect(Collectors.toSet());
        List<Map<String, String>> skuList = drugEntDrugtableService.listDrugs(skuIds);
        Map<Integer, Map<String, String>> skuMap = skuList.stream().collect(Collectors.toMap(sku -> Integer.valueOf(sku.get("drug_id")), Function.identity()));


        // 封装返回结果
        Page<Tree<Integer>> newPage = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        List<TreeNode<Integer>> toPages = getToPages(records, consigneeMap, skuMap);

        List<Tree<Integer>> builds = new ArrayList<>();
        // 事业部 || 区域 || 平台
        if (entLevel == 0 || entLevel == 1 || entLevel == -1) {
            List<Tree<Integer>> build = TreeUtil.build(toPages, 0);
            builds.addAll(build);
        }
        // 服务商
        else {

            for (TreeNode<Integer> node : toPages) {
                List<Tree<Integer>> build = TreeUtil.build(Collections.singletonList(node), node.getParentId());
                builds.addAll(build);
            }
        }


        newPage.setRecords(builds);

        return newPage;
    }

    private List<TreeNode<Integer>> getToPages(List<SysImplementPlan> records, Map<Integer, SysDept> consigneeMap, Map<Integer, Map<String, String>> skuMap) {
        return records.stream().map(record -> {
            SysImplementPlanVO.ToPage p = BeanUtil.copyProperties(record, SysImplementPlanVO.ToPage.class);

            // 设置承接人信息
            Map<String, Object> consigneeInfo = new HashMap<>(3);
            consigneeInfo.put("consigneeId", record.getConsigneeId());
            SysDept sysDept = consigneeMap.get(record.getConsigneeId());
            consigneeInfo.put("consigneeName", sysDept == null ? "" : sysDept.getName());
            p.setConsigneeInfo(consigneeInfo);

            // 执行包总值
            Set<String> pkgIds = planPkgService.list(Wrappers.<SysPlanPkg>lambdaQuery().eq(SysPlanPkg::getPlanId, record.getPlanId())).stream().map(rel -> rel.getPkgId().toString()).collect(Collectors.toSet());
            long execScoreVal = 0L;
            if (CollUtil.isNotEmpty(pkgIds)) {
                List<WmScorePackage> packages = scorePackageService.list(Wrappers.<WmScorePackage>lambdaQuery().in(WmScorePackage::getRelationScoreId, pkgIds));
                if (CollUtil.isNotEmpty(packages)) {
                    execScoreVal = packages.stream().mapToLong(WmScorePackage::getScore).sum();
                }

            }
            p.setExecScoreValue(execScoreVal);

            // 产品信息
            Map<String, String> sku = skuMap.get(record.getSkuId());
            Map<String, Object> skuInfo = new HashMap<>(3);
            skuInfo.put("skuId", record.getSkuId());
            skuInfo.put("skuName", MapUtil.isEmpty(sku) ? "" : sku.get("drug_name"));
            skuInfo.put("manufacturer", MapUtil.isEmpty(sku) ? "" : sku.get("ent_name"));
            p.setSkuInfo(skuInfo);

            // 获取计划详情
            List<SysImplPlanDetails> details = implPlanDetailsService.list(Wrappers.<SysImplPlanDetails>lambdaQuery()
                    .eq(SysImplPlanDetails::getPlanId, record.getPlanId())
                    .orderByDesc(SysImplPlanDetails::getUpdateTime)
            );
            p.setPlanDetails(CollUtil.isEmpty(details) ? null : details.get(0));

            TreeNode<Integer> treeNode = new TreeNode<>();
            treeNode.setId(p.getPlanId());
            treeNode.setParentId(p.getParentId());
            treeNode.setName(p.getPlanName());
            treeNode.setWeight(p.getPlanId());
            Map<String, Object> extra = BeanUtil.beanToMap(p);
            treeNode.setExtra(extra);

            return treeNode;
        }).collect(Collectors.toList());
    }

    private Page<SysImplementPlan> getPage(HnqzUser user,
                                           int entLevel,
                                           Integer current,
                                           Integer size,
                                           String planName,
                                           Integer consigneeId,
                                           Integer consignorId,
                                           LocalDate[] period) {

        LambdaQueryWrapper<SysImplementPlan> wrapper = Wrappers.<SysImplementPlan>lambdaQuery()
                .like(StrUtil.isNotBlank(planName), SysImplementPlan::getPlanName, planName);

        // 服务商管理员（只查询发给我的）
        if (entLevel == 2) {
            wrapper.eq(SysImplementPlan::getPlanState, UpmsState.ImplementPlanState.PASSED)
                    .eq(SysImplementPlan::getConsigneeId, user.getDeptId());
        }
        // 区域 (查询发给我的和我下发的）
        else if (entLevel == 1) {
            wrapper.eq(SysImplementPlan::getPlanState, UpmsState.ImplementPlanState.PASSED)
                    .and(wrp -> wrp.eq(SysImplementPlan::getConsigneeId, user.getDeptId()).or().eq(SysImplementPlan::getConsignorId, user.getDeptId()));
        }
        // 事业部管理员||药企（只查询我下发的）
        else if (entLevel == -1) {
            wrapper.eq(SysImplementPlan::getConsignorId, user.getDeptId())
                    .eq(SysImplementPlan::getParentId, 0);
        }
        // 平台||营销中心（查询全部）
        else {
            wrapper.eq(SysImplementPlan::getParentId, 0);
        }

        if (ArrayUtil.isNotEmpty(period) && period.length == 2) {
            wrapper.between(SysImplementPlan::getCreateTime, period[0], period[1]);
        }

        return implementPlanService.page(new Page<>(current, size), wrapper);
    }

    /**
     * 审核计划
     *
     * @param resource 审核信息
     * @param user     操作人
     * @return 审核结果
     */
    public Boolean checkPlan(SysImplementPlanDTO.OnCheck resource, HnqzUser user) {


        implementPlanService.checkState(resource.getPlanId(), Boolean.TRUE, Collections.singletonList(UpmsState.ImplementPlanState.INIT));

        Boolean result = resource.getResult();
        String username = user.getUsername();
        LocalDateTime now = LocalDateTime.now();
        SysImplementPlan plan = new SysImplementPlan();
        plan.setPlanId(resource.getPlanId());
        plan.setPlanState(result ? UpmsState.ImplementPlanState.PASSED : UpmsState.ImplementPlanState.REJECTED);
        plan.setCheckResult(result);
        plan.setCheckMessage(resource.getMsg());
        plan.setCheckTime(now);
        plan.setChecker(username);
        plan.setUpdateBy(username);
        plan.setUpdateTime(now);

        // 归还父计划的分值
        if (!result) {
            SysImplementPlan p = implementPlanService.getById(resource.getPlanId());
            if (p.getParentId() != 0) {
                SysImplementPlan parentPlan = implementPlanService.getById(p.getParentId());
                parentPlan.setAvailScore(parentPlan.getAvailScore() + p.getPlanScore());
                implementPlanService.updatePlan(parentPlan);
            }
        }


        return implementPlanService.updatePlan(plan);
    }

    /**
     * 创建计划详情
     *
     * @param resource 计划详情
     * @param user     操作人
     * @return 创建结果
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean createPlanDetails(SysImplementPlanDetailsDTO.OnCreate resource, HnqzUser user) {

        LocalDateTime now = LocalDateTime.now();
        String username = user.getUsername();
        List<SysImplementPlanDetailsDTO.OnItemCreate> items = resource.getItems();


        SysImplPlanDetails details = new SysImplPlanDetails();
        details.setPlanId(resource.getPlanId());
        details.setQty(resource.getQty());
        details.setTotal(resource.getTotal());
        details.setDetailsState(UpmsState.ImplPlanDetailsState.INIT);
        details.setCreateBy(username);
        details.setUpdateBy(username);
        details.setCreateTime(now);
        details.setUpdateTime(now);
        implPlanDetailsService.createDetails(details);

        Integer dId = details.getDetailsId();
        List<SysImplPlanDetailsItem> detailsItems = items.stream().map(item -> {
            SysImplPlanDetailsItem i = new SysImplPlanDetailsItem();
            i.setDetailsId(dId);
            i.setTaskTypeId(item.getTaskTypeId());
            i.setQty(item.getQty());
            i.setTaskTypeName(item.getTaskTypeName());
            i.setScore(item.getScore());
            i.setSubtotal(item.getSubtotal());
            return i;
        }).collect(Collectors.toList());

        implPlanDetailsItemService.saveItems(detailsItems);

        return Boolean.TRUE;
    }

    /**
     * 审核计划详情
     *
     * @param resource 计划详情内容
     * @param user     操作人
     * @return 审核结果
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean checkPlanDetails(SysImplementPlanDetailsDTO.OnCheck resource, HnqzUser user) {

        Boolean result = resource.getResult();
        String username = user.getUsername();
        LocalDateTime now = LocalDateTime.now();
        SysImplPlanDetails details = new SysImplPlanDetails();
        details.setDetailsId(resource.getDetailsId());

        switch (resource.getCurrentState()) {
            case INIT:
                details.setDetailsState(result ? UpmsState.ImplPlanDetailsState.CKT1 : UpmsState.ImplPlanDetailsState.REJECTED);
                details.setCktRes1(result);
                details.setCktMsg1(resource.getMsg());
                details.setCktTime1(now);
                details.setCktName1(username);
                break;

            case CKT1:
                details.setDetailsState(result ? UpmsState.ImplPlanDetailsState.CKT2 : UpmsState.ImplPlanDetailsState.REJECTED);
                details.setCktRes2(result);
                details.setCktMsg2(resource.getMsg());
                details.setCktTime2(now);
                details.setCktName2(username);
                break;
            case CKT2:
                details.setDetailsState(result ? UpmsState.ImplPlanDetailsState.PASSED : UpmsState.ImplPlanDetailsState.REJECTED);
                details.setCktRes3(result);
                details.setCktMsg3(resource.getMsg());
                details.setCktTime3(now);
                details.setCktName3(username);
                break;
            default:
                throw new BizException("当前状态不支持审核");

        }
        details.setUpdateBy(username);
        details.setUpdateTime(now);

        return implPlanDetailsService.updateDetails(details);
    }

    public Map<String, Object> getPlanDetails(Integer detailsId) {
        SysImplPlanDetails details = implPlanDetailsService.getById(detailsId);
        Map<String, Object> detailsMap;
        if (details != null) {
            detailsMap = BeanUtil.beanToMap(details);

            List<SysImplPlanDetailsItem> items = implPlanDetailsItemService.listDetailItems(details.getDetailsId());

            detailsMap.put("items", CollUtil.isNotEmpty(items) ? items : Collections.emptyList());


        } else {
            detailsMap = Collections.emptyMap();
        }

        return detailsMap;
    }

    public List<SysImplementPlanVO.ToList> listPlans(UpmsState.ImplementPlanState planState, HnqzUser user) {

        List<SysImplementPlan> plans = implementPlanService.list(Wrappers.<SysImplementPlan>lambdaQuery()
                .eq(SysImplementPlan::getConsignorId, user.getDeptId())
                .eq(SysImplementPlan::getPlanState, planState));


        if (CollUtil.isEmpty(plans)) {
            return Collections.emptyList();
        }


        // 获取承接人ID
        Set<Integer> consigneeIds = plans.stream().mapToInt(SysImplementPlan::getConsigneeId).boxed().collect(Collectors.toSet());
        // 获取承接人
        Map<Integer, SysDept> consigneeMap = deptService.mapConsignee(consigneeIds);
        // 获取商品信息
        Set<Integer> skuIds = plans.stream().map(SysImplementPlan::getSkuId).collect(Collectors.toSet());
        List<Map<String, String>> skuList = drugEntDrugtableService.listDrugs(skuIds);
        Map<Integer, Map<String, String>> skuMap = skuList.stream().collect(Collectors.toMap(sku -> Integer.valueOf(sku.get("drug_id")), Function.identity()));


        return plans.stream().map(p -> {
            SysImplementPlanVO.ToList copied = BeanUtil.copyProperties(p, SysImplementPlanVO.ToList.class);

            // 设置承接人信息
            Map<String, Object> consigneeInfo = new HashMap<>(2);
            consigneeInfo.put("consigneeId", p.getConsigneeId());
            SysDept sysDept = consigneeMap.get(p.getConsigneeId());
            consigneeInfo.put("consigneeName", sysDept == null ? "" : sysDept.getName());
            copied.setConsigneeInfo(consigneeInfo);

            // 产品信息
            Map<String, String> sku = skuMap.get(p.getSkuId());
            Map<String, Object> skuInfo = new HashMap<>(3);
            skuInfo.put("skuId", p.getSkuId());
            skuInfo.put("skuName", MapUtil.isEmpty(sku) ? "" : sku.get("drug_name"));
            skuInfo.put("manufacturer", MapUtil.isEmpty(sku) ? "" : sku.get("ent_name"));
            copied.setSkuInfo(skuInfo);

            return copied;

        }).collect(Collectors.toList());

    }
}
