package com.qunzhixinxi.hnqz.admin.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsState;
import com.qunzhixinxi.hnqz.admin.api.entity.SysImplementPlan;
import com.qunzhixinxi.hnqz.admin.mapper.SysImplementPlanMapper;
import com.qunzhixinxi.hnqz.admin.service.SysImplementPlanService;
import com.qunzhixinxi.hnqz.common.core.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;

/**
 * 执行计划服务实现
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2024/05/19 11:14
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysImplementPlanServiceImpl extends ServiceImpl<SysImplementPlanMapper, SysImplementPlan> implements SysImplementPlanService {

    /**
     * 创建计划
     *
     * @param plan 计划信息
     * @return 创建结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean createPlan(SysImplementPlan plan) {
        return this.save(plan);
    }

    /**
     * 更新计划
     *
     * @param plan 计划
     * @return 更新结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updatePlan(SysImplementPlan plan) {
        return this.updateById(plan);
    }

    /**
     * 校验状态
     *
     * @param planId              计划ID
     * @param res                 校验结果
     * @param implementPlanStates 目标状态
     */
    @Override
    public void checkState(Integer planId, Boolean res, Collection<UpmsState.ImplementPlanState> implementPlanStates) {
        int count = this.count(Wrappers.<SysImplementPlan>lambdaQuery().eq(SysImplementPlan::getPlanId, planId).in(SysImplementPlan::getPlanState, implementPlanStates));
        if (!res.equals(count != 0)) {
            throw new BizException("当前计划状态不支持操作");
        }
    }

    /**
     * 可用分
     *
     * @param planId    计划ID
     * @param planScore 目标值
     * @return 可用分
     */
    @Override
    public Long calcAvailScore(Integer planId, Long planScore) {
        SysImplementPlan plan = this.getById(planId);

        long availScore = plan.getAvailScore() - planScore;

        if (availScore < 0){
            throw new BizException("可用值不足");
        }
        return availScore;
    }
}




